use nom::multi::many_till;

use crate::cpu::instructions::Instruction;
use crate::interpreter::parser::instructions::parse_line;
use crate::interpreter::parser::primitives::test_line_end;
use crate::msl_result::{MSLError, MSLResult};

#[derive(Debug)]
pub enum CodeLine {
    Comment(String),
    Code(Instruction),
}

pub mod definitions;
pub mod instructions;
pub mod primitives;

pub fn parse_code(input: String) -> MSLResult<Vec<Instruction>> {
    let res = many_till(parse_line, test_line_end)(input.as_str());
    if let Ok((_, (codelines, _))) = res {
        let code = codelines
            .into_iter()
            .filter_map(|e| {
                if let Some(CodeLine::Code(inst)) = e {
                    Some(inst)
                } else {
                    None
                }
            })
            .collect::<Vec<_>>();
        Ok(code)
    } else {
        let description = match res.err().unwrap() {
            nom::Err::Incomplete(d) => format!("Incomplete input. {:?}", d),
            nom::Err::Error(verbose) => {
                nom::error::convert_error(input.as_str(), verbose)
            }
            nom::Err::Failure(verbose) => {
                nom::error::convert_error(input.as_str(), verbose)
            }
        };
        Err(MSLError::InterpreterError { desc: description }.into())
    }
}
