use crate::cpu::instructions::Instruction;
use crate::msl_result::{MSLError, MSLResult};
use crate::ram::model::RamValue;

use super::model::RAM;

impl RAM {
    pub fn new(size: usize) -> RAM {
        RAM {
            mem: vec![RamValue::EMPTY; size],
            ..Default::default()
        }
    }

    pub fn access_data(&mut self, address: usize) -> MSLResult<&RamValue> {
        let val = self.mem.get(address);
        if let Some(val) = val {
            Ok(val)
        } else {
            Err(MSLError::MemoryError(format!(
                "Attempt to access invalid address ({})",
                address
            ))
                .into())
        }
    }

    pub fn store_data(
        &mut self,
        address: usize,
        data: RamValue,
    ) -> MSLResult<()> {
        if address < self.mem.len() {
            self.mem[address] = data;
            Ok(())
        } else {
            Err(MSLError::MemoryError(format!(
                "Attempt to access invalid address ({})",
                address
            ))
                .into())
        }
    }

    pub fn store_instructions(&mut self, instructions: Vec<Instruction>) -> MSLResult<()> {
        for (i, item) in instructions.into_iter().enumerate() {
            self.store_data(i, RamValue::INSTR(item))?;
        }
        Ok(())
    }
}
